package java.util.stream;

import java.util.Iterator;
import java.util.Objects;
import java.util.function.UnaryOperator;
import java.util.Arrays;
import java.util.Spliterators;
import java.util.function.BiConsumer;
import java.util.function.Supplier;
import java.util.function.BiFunction;
import java.util.Optional;
import java.util.function.BinaryOperator;
import java.util.function.IntFunction;
import java.util.function.Consumer;
import java.util.Comparator;
import java.util.function.ToDoubleFunction;
import java.util.function.ToLongFunction;
import java.util.function.ToIntFunction;
import java.util.function.Function;
import java.util.function.Predicate;

public interface Stream<T> extends BaseStream<T, Stream<T>>
{
    public abstract Stream<T> filter(Predicate<? super T> p0);
    
    public abstract <R> Stream<R> map(Function<? super T, ? extends R> p0);
    
    public abstract IntStream mapToInt(ToIntFunction<? super T> p0);
    
    public abstract LongStream mapToLong(ToLongFunction<? super T> p0);
    
    public abstract DoubleStream mapToDouble(ToDoubleFunction<? super T> p0);
    
    public abstract <R> Stream<R> flatMap(Function<? super T, ? extends Stream<? extends R>> p0);
    
    public abstract IntStream flatMapToInt(Function<? super T, ? extends IntStream> p0);
    
    public abstract LongStream flatMapToLong(Function<? super T, ? extends LongStream> p0);
    
    public abstract DoubleStream flatMapToDouble(Function<? super T, ? extends DoubleStream> p0);
    
    public abstract Stream<T> distinct();
    
    public abstract Stream<T> sorted();
    
    public abstract Stream<T> sorted(Comparator<? super T> p0);
    
    public abstract Stream<T> peek(Consumer<? super T> p0);
    
    public abstract Stream<T> limit(long p0);
    
    public abstract Stream<T> skip(long p0);
    
    public abstract void forEach(Consumer<? super T> p0);
    
    public abstract void forEachOrdered(Consumer<? super T> p0);
    
    public abstract Object[] toArray();
    
    public abstract <A> A[] toArray(IntFunction<A[]> p0);
    
    public abstract T reduce(T p0, BinaryOperator<T> p1);
    
    public abstract Optional<T> reduce(BinaryOperator<T> p0);
    
    public abstract <U> U reduce(U p0, BiFunction<U, ? super T, U> p1, BinaryOperator<U> p2);
    
    public abstract <R> R collect(Supplier<R> p0, BiConsumer<R, ? super T> p1, BiConsumer<R, R> p2);
    
    public abstract <R, A> R collect(Collector<? super T, A, R> p0);
    
    public abstract Optional<T> min(Comparator<? super T> p0);
    
    public abstract Optional<T> max(Comparator<? super T> p0);
    
    public abstract long count();
    
    public abstract boolean anyMatch(Predicate<? super T> p0);
    
    public abstract boolean allMatch(Predicate<? super T> p0);
    
    public abstract boolean noneMatch(Predicate<? super T> p0);
    
    public abstract Optional<T> findFirst();
    
    public abstract Optional<T> findAny();
    
    public static <T> Stream.Builder<T> builder() {
        return new Streams.StreamBuilderImpl();
    }
    
    public static <T> Stream<T> empty() {
        return StreamSupport.stream(Spliterators.emptySpliterator(), false);
    }
    
    public static <T> Stream<T> of(T t) {
        return StreamSupport.stream(new Streams.StreamBuilderImpl(t), false);
    }
    
    @SafeVarargs
    public static <T> Stream<T> of(T... array) {
        return Arrays.stream(array);
    }
    
    public static <T> Stream<T> iterate(final T t, final UnaryOperator<T> obj) {
        Objects.requireNonNull(obj);
        return StreamSupport.stream(Spliterators.spliteratorUnknownSize(new Iterator<T>() {
            T t = Streams.NONE;
            
            @Override
            public boolean hasNext() {
                return true;
            }
            
            @Override
            public T next() {
                return this.t = ((this.t == Streams.NONE) ? t : obj.apply(this.t));
            }
        }, 1040), false);
    }
    
    public static <T> Stream<T> generate(Supplier<T> obj) {
        Objects.requireNonNull(obj);
        return StreamSupport.stream(new StreamSpliterators.InfiniteSupplyingSpliterator.OfRef(Long.MAX_VALUE, obj), false);
    }
    
    public static <T> Stream<T> concat(Stream<? extends T> obj, Stream<? extends T> obj2) {
        Objects.requireNonNull(obj);
        Objects.requireNonNull(obj2);
        return (Stream)StreamSupport.stream(new Streams.ConcatSpliterator.OfRef(obj.spliterator(), obj2.spliterator()), obj.isParallel() || obj2.isParallel()).onClose(Streams.composedClose(obj, obj2));
    }
    
    public interface Builder<T> extends Consumer<T>
    {
        public abstract void accept(T p0);
        
        default public Stream.Builder<T> add(T t) {
            this.accept(t);
            return this;
        }
        
        public abstract Stream<T> build();
    }
}
